@@ -1,9 +1,16 @@
-#include "Rstats.h"
+#include "Rstats_ElementFunc.h"
 
 // Rstats::ElementFunc
 namespace Rstats {
   namespace ElementFunc {
-
+    
+    /* Note:
+      In ElementFunction, croak method should't be called.
+      If you want to tell warnings, use Rstats::add_warn function.
+      Rstats::clear_warn function should't be called.
+      If you want to tell NA introduced, throw Rstats::NaException.
+    */
+    
     // add
     Rstats::Complex add(Rstats::Complex e1, Rstats::Complex e2) { return e1 + e2; }
     Rstats::Double add(Rstats::Double e1, Rstats::Double e2) { return e1 + e2; }
@@ -49,9 +56,6 @@ namespace Rstats {
     }
 
     // remainder
-    Rstats::Complex remainder(Rstats::Complex e1, Rstats::Complex e2) {
-      croak("unimplemented complex operation(Rstats::VectorFunc::remainder())");
-    }
     Rstats::Double remainder(Rstats::Double e1, Rstats::Double e2) {
       if (Rstats::Util::is_NaN(e1) || Rstats::Util::is_NaN(e2) || e2 == 0) {
         return Rstats::Util::NaN();
@@ -173,7 +177,6 @@ namespace Rstats {
     Rstats::Double log2(Rstats::Logical e1) { return Rstats::ElementFunc::log2((Rstats::Double)e1); }
     
     // expm1
-    Rstats::Complex expm1(Rstats::Complex e1) { croak("Error in expm1 : unimplemented complex function"); }
     Rstats::Double expm1(Rstats::Double e1) { return ::expm1(e1); }
     Rstats::Double expm1(Rstats::Integer e1) { return Rstats::ElementFunc::expm1((Rstats::Double)e1); }
     Rstats::Double expm1(Rstats::Logical e1) { return Rstats::ElementFunc::expm1((Rstats::Double)e1); }
@@ -415,7 +418,7 @@ namespace Rstats {
     Rstats::Double acosh(Rstats::Double e1) {
       if (e1 >= 1) {
         if (Rstats::Util::is_Inf(e1)) {
-          Rstats::Util::add_warn(Rstats::WARN_NAN_PRODUCED);
+          Rstats::add_warn(Rstats::WARN_NAN_PRODUCED);
           return Rstats::Util::NaN();
         }
         else {
@@ -427,7 +430,7 @@ namespace Rstats {
         }
       }
       else {
-        Rstats::Util::add_warn(Rstats::WARN_NAN_PRODUCED);
+        Rstats::add_warn(Rstats::WARN_NAN_PRODUCED);
         return Rstats::Util::NaN();
       }
     }
@@ -437,11 +440,11 @@ namespace Rstats {
     // atanh
     Rstats::Complex atanh(Rstats::Complex e1) {
       if (e1 == Rstats::Complex(1, 0)) {
-        Rstats::Util::add_warn(Rstats::WARN_NAN_PRODUCED);
+        Rstats::add_warn(Rstats::WARN_NAN_PRODUCED);
         return Rstats::Complex(Rstats::Util::Inf(), Rstats::Util::NaN());
       }
       else if (e1 == Rstats::Complex(-1, 0)) {
-        Rstats::Util::add_warn(Rstats::WARN_NAN_PRODUCED);
+        Rstats::add_warn(Rstats::WARN_NAN_PRODUCED);
         return Rstats::Complex(-Rstats::Util::Inf(), Rstats::Util::NaN());
       }
       else {
@@ -456,7 +459,7 @@ namespace Rstats {
     }
     Rstats::Double atanh(Rstats::Double e1) {
       if (Rstats::Util::is_Inf(e1)) {
-        Rstats::Util::add_warn(Rstats::WARN_NAN_PRODUCED);
+        Rstats::add_warn(Rstats::WARN_NAN_PRODUCED);
         return Rstats::Util::NaN();
       }
       else {
@@ -470,7 +473,7 @@ namespace Rstats {
           return std::log((1 + e1) / (1 - e1)) / 2;
         }
         else {
-          Rstats::Util::add_warn(Rstats::WARN_NAN_PRODUCED);
+          Rstats::add_warn(Rstats::WARN_NAN_PRODUCED);
           return Rstats::Util::NaN();
         }
       }
@@ -523,7 +526,6 @@ namespace Rstats {
     }
 
     // And
-    Rstats::Logical And(Rstats::Character e1, Rstats::Character e2) { croak("operations are possible only for numeric, logical or complex types"); }
     Rstats::Logical And(Rstats::Complex e1, Rstats::Complex e2) {
       if (e1 != Rstats::Complex(0, 0) && e2 != Rstats::Complex(0, 0)) { return 1; }
       else { return 0; }
@@ -538,7 +540,6 @@ namespace Rstats {
     }
 
     // Or
-    Rstats::Logical Or(Rstats::Character e1, Rstats::Character e2) { croak("operations are possible only for numeric, logical or complex types"); }
     Rstats::Logical Or(Rstats::Complex e1, Rstats::Complex e2) {
       if (e1 != Rstats::Complex(0, 0) || e2 != Rstats::Complex(0, 0)) { return 1; }
       else { return 0; }
@@ -747,7 +748,7 @@ namespace Rstats {
     }
     Rstats::Double as_double(Rstats::Complex e1) {
       if (e1.imag() != 0) {
-        Rstats::Util::add_warn(Rstats::WARN_IMAGINARY_PART_DISCARDED);
+        Rstats::add_warn(Rstats::WARN_IMAGINARY_PART_DISCARDED);
       }
       return e1.real();
     }
@@ -763,13 +764,13 @@ namespace Rstats {
         return value;
       }
       else {
-        Rstats::Util::add_warn(WARN_NA_INTRODUCED);
+        Rstats::add_warn(WARN_NA_INTRODUCED);
         throw Rstats::NaException;
       }
     }
     Rstats::Integer as_integer(Rstats::Complex e1) {
       if (Rstats::Util::is_NaN(e1.real()) || Rstats::Util::is_NaN(e1.imag()) || Rstats::Util::is_Inf(e1.real()) || Rstats::Util::is_Inf(e1.imag())) {
-        Rstats::Util::add_warn(WARN_NA_INTRODUCED);
+        Rstats::add_warn(WARN_NA_INTRODUCED);
         throw Rstats::NaException;
       }
       else {
@@ -778,7 +779,7 @@ namespace Rstats {
     }
     Rstats::Integer as_integer(Rstats::Double e1) {
       if (Rstats::Util::is_NaN(e1) || Rstats::Util::is_Inf(e1)) {
-        Rstats::Util::add_warn(WARN_NA_INTRODUCED);
+        Rstats::add_warn(WARN_NA_INTRODUCED);
         throw Rstats::NaException;
       }
       else {
@@ -800,13 +801,13 @@ namespace Rstats {
         }
       }
       else {
-        Rstats::Util::add_warn(WARN_NA_INTRODUCED);
+        Rstats::add_warn(WARN_NA_INTRODUCED);
         throw Rstats::NaException;
       }
     }
     Rstats::Logical as_logical(Rstats::Complex e1) {
       if (Rstats::Util::is_NaN(e1.real()) || Rstats::Util::is_NaN(e1.imag())) {
-        Rstats::Util::add_warn(WARN_NA_INTRODUCED);
+        Rstats::add_warn(WARN_NA_INTRODUCED);
         throw Rstats::NaException;
       }
       else if (e1.real() || e1.imag()) {
@@ -818,7 +819,7 @@ namespace Rstats {
     }
     Rstats::Logical as_logical(Rstats::Double e1) {
       if (Rstats::Util::is_NaN(e1)) {
-        Rstats::Util::add_warn(WARN_NA_INTRODUCED);
+        Rstats::add_warn(WARN_NA_INTRODUCED);
         throw Rstats::NaException;
       }
       else {
